@@ -15,15 +15,14 @@
 # limitations under the License.
 
 # to configure behavior, define $CQL_TEST_HOST to the destination address
-# for Thrift connections, and $CQL_TEST_PORT to the associated port.
+# and $CQL_TEST_PORT to the associated port.
 
-from __future__ import with_statement
 
 import locale
 import os
 import re
-from .basecase import BaseTestCase, cqlsh
-from .cassconnect import testrun_cqlsh
+from .basecase import BaseTestCase, cqlsh, cqlshlog
+from .cassconnect import create_db, remove_db, testrun_cqlsh
 from .run_cqlsh import TimeoutError
 import unittest
 import sys
@@ -44,6 +43,14 @@ completion_separation_re = re.compile(r'\s+')
 @unittest.skipIf(sys.platform == "win32", 'Tab completion tests not supported on Windows')
 class CqlshCompletionCase(BaseTestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        create_db()
+
+    @classmethod
+    def tearDownClass(cls):
+        remove_db()
+
     def setUp(self):
         env = os.environ.copy()
         env['COLUMNS'] = '100000'
@@ -88,14 +95,14 @@ class CqlshCompletionCase(BaseTestCase):
             prompt_regex = self.cqlsh.prompt.lstrip() + re.escape(inputstring)
             msg = ('Double-tab completion '
                    'does not print prompt for input "{}"'.format(inputstring))
-            self.assertRegexpMatches(choice_lines[-1], prompt_regex, msg=msg)
+            self.assertRegex(choice_lines[-1], prompt_regex, msg=msg)
 
         choice_lines = [line.strip() for line in choice_lines[:-1]]
         choice_lines = [line for line in choice_lines if line]
 
         if split_completed_lines:
-            completed_lines = map(set, (completion_separation_re.split(line.strip())
-                                  for line in choice_lines))
+            completed_lines = list(map(set, (completion_separation_re.split(line.strip())
+                                  for line in choice_lines)))
 
             if not completed_lines:
                 return set()
@@ -146,7 +153,7 @@ class CqlshCompletionCase(BaseTestCase):
                 # retry once
                 self.cqlsh.send(CTRL_C)
                 self.cqlsh.read_to_next_prompt(timeout=10.0)
-
+ 
 
     def strategies(self):
         return self.module.CqlRuleSet.replication_strategies
@@ -358,8 +365,6 @@ class TestCqlshCompletion(CqlshCompletionCase):
                             choices=[',', 'WHERE'])
         self.trycompletions("UPDATE empty_table SET lonelycol = 'eggs' WHERE ",
                             choices=['TOKEN(', 'lonelykey'])
-        self.trycompletions("UPDATE empty_table SET lonelycol = 'eggs' WHERE ",
-                            choices=['TOKEN(', 'lonelykey'])
 
         self.trycompletions("UPDATE empty_table SET lonelycol = 'eggs' WHERE lonel",
                             immediate='ykey ')
@@ -405,7 +410,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'twenty_rows_composite_table',
                                      'utf8_with_special_chars',
                                      'system_traces.', 'songs',
-                                     '"' + self.cqlsh.keyspace + '".'],
+                                     self.cqlsh.keyspace + '.'],
                             other_choices_ok=True)
 
         self.trycompletions('DELETE FROM ',
@@ -420,7 +425,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'system_traces.', 'songs',
                                      'system_auth.', 'system_distributed.',
                                      'system_schema.', 'system_traces.',
-                                     '"' + self.cqlsh.keyspace + '".'],
+                                     self.cqlsh.keyspace + '.'],
                             other_choices_ok=True)
         self.trycompletions('DELETE FROM twenty_rows_composite_table ',
                             choices=['USING', 'WHERE'])
@@ -542,13 +547,13 @@ class TestCqlshCompletion(CqlshCompletionCase):
         self.trycompletions('DROP K', immediate='EYSPACE ')
         quoted_keyspace = '"' + self.cqlsh.keyspace + '"'
         self.trycompletions('DROP KEYSPACE ',
-                            choices=['IF', quoted_keyspace])
+                            choices=['IF', self.cqlsh.keyspace])
 
         self.trycompletions('DROP KEYSPACE ' + quoted_keyspace,
                             choices=[';'])
 
         self.trycompletions('DROP KEYSPACE I',
-                            immediate='F EXISTS ' + quoted_keyspace + ';')
+                            immediate='F EXISTS ' + self.cqlsh.keyspace + ' ;')
 
     def create_columnfamily_table_template(self, name):
         """Parameterized test for CREATE COLUMNFAMILY and CREATE TABLE. Since
@@ -557,11 +562,11 @@ class TestCqlshCompletion(CqlshCompletionCase):
         prefix = 'CREATE ' + name + ' '
         quoted_keyspace = '"' + self.cqlsh.keyspace + '"'
         self.trycompletions(prefix + '',
-                            choices=['IF', quoted_keyspace, '<new_table_name>'])
+                            choices=['IF', self.cqlsh.keyspace, '<new_table_name>'])
         self.trycompletions(prefix + 'IF ',
                             immediate='NOT EXISTS ')
         self.trycompletions(prefix + 'IF NOT EXISTS ',
-                            choices=['<new_table_name>', quoted_keyspace])
+                            choices=['<new_table_name>', self.cqlsh.keyspace])
         self.trycompletions(prefix + 'IF NOT EXISTS new_table ',
                             immediate='( ')
 
@@ -602,23 +607,21 @@ class TestCqlshCompletion(CqlshCompletionCase):
         self.trycompletions(prefix + ' new_table (col_a int PRIMARY KEY) WITH ',
                             choices=['bloom_filter_fp_chance', 'compaction',
                                      'compression',
-                                     'dclocal_read_repair_chance',
                                      'default_time_to_live', 'gc_grace_seconds',
                                      'max_index_interval',
                                      'memtable_flush_period_in_ms',
-                                     'read_repair_chance', 'CLUSTERING',
+                                     'CLUSTERING',
                                      'COMPACT', 'caching', 'comment',
-                                     'min_index_interval', 'speculative_retry', 'cdc'])
+                                     'min_index_interval', 'speculative_retry', 'additional_write_policy', 'cdc', 'read_repair'])
         self.trycompletions(prefix + ' new_table (col_a int PRIMARY KEY) WITH ',
                             choices=['bloom_filter_fp_chance', 'compaction',
                                      'compression',
-                                     'dclocal_read_repair_chance',
                                      'default_time_to_live', 'gc_grace_seconds',
                                      'max_index_interval',
                                      'memtable_flush_period_in_ms',
-                                     'read_repair_chance', 'CLUSTERING',
+                                     'CLUSTERING',
                                      'COMPACT', 'caching', 'comment',
-                                     'min_index_interval', 'speculative_retry', 'cdc'])
+                                     'min_index_interval', 'speculative_retry', 'additional_write_policy', 'cdc', 'read_repair'])
         self.trycompletions(prefix + ' new_table (col_a int PRIMARY KEY) WITH bloom_filter_fp_chance ',
                             immediate='= ')
         self.trycompletions(prefix + ' new_table (col_a int PRIMARY KEY) WITH bloom_filter_fp_chance = ',
@@ -661,13 +664,12 @@ class TestCqlshCompletion(CqlshCompletionCase):
                             + "{'class': 'SizeTieredCompactionStrategy'} AND ",
                             choices=['bloom_filter_fp_chance', 'compaction',
                                      'compression',
-                                     'dclocal_read_repair_chance',
                                      'default_time_to_live', 'gc_grace_seconds',
                                      'max_index_interval',
                                      'memtable_flush_period_in_ms',
-                                     'read_repair_chance', 'CLUSTERING',
+                                     'CLUSTERING',
                                      'COMPACT', 'caching', 'comment',
-                                     'min_index_interval', 'speculative_retry', 'cdc'])
+                                     'min_index_interval', 'speculative_retry', 'additional_write_policy', 'cdc', 'read_repair'])
         self.trycompletions(prefix + " new_table (col_a int PRIMARY KEY) WITH compaction = "
                             + "{'class': 'DateTieredCompactionStrategy', '",
                             choices=['base_time_seconds', 'max_sstable_age_days',
@@ -705,10 +707,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
         self.trycompletions('CREATE TA', immediate='BLE ')
         self.create_columnfamily_table_template('TABLE')
 
-    def test_complete_in_describe(self):
-        """
-        Tests for Cassandra-10733
-        """
+    def test_complete_in_describe(self):  # Cassandra-10733
         self.trycompletions('DES', immediate='C')
         # quoted_keyspace = '"' + self.cqlsh.keyspace + '"'
         self.trycompletions('DESCR', immediate='IBE ')
@@ -723,7 +722,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'utf8_with_special_chars',
                                      'system_traces.', 'songs',
                                      'system_distributed.',
-                                     '"' + self.cqlsh.keyspace + '".'],
+                                     self.cqlsh.keyspace + '.'],
                             other_choices_ok=True)
 
         self.trycompletions('DESC TYPE ',
@@ -746,7 +745,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'fbestsong',
                                      'fmax',
                                      'fmin',
-                                     '"' + self.cqlsh.keyspace + '".'],
+                                     self.cqlsh.keyspace + '.'],
                             other_choices_ok=True)
 
         self.trycompletions('DESC AGGREGATE ',
@@ -756,7 +755,7 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'system_distributed.',
                                      'aggmin',
                                      'aggmax',
-                                     '"' + self.cqlsh.keyspace + '".'],
+                                     self.cqlsh.keyspace + '.'],
                             other_choices_ok=True)
 
         # Unfortunately these commented tests will not work. This is due to the keyspace name containing quotes;
@@ -812,20 +811,29 @@ class TestCqlshCompletion(CqlshCompletionCase):
                                      'aggmax'],
                             other_choices_ok=True)
 
-    def test_complete_in_drop_columnfamily(self):
-        pass
+    # TODO: CASSANDRA-16640
+    # def test_complete_in_drop_columnfamily(self):
+    #     pass
+    #
+    # def test_complete_in_truncate(self):
+    #     pass
+    #
+    # def test_complete_in_alter_columnfamily(self):
+    #     pass
+    #
+    # def test_complete_in_use(self):
+    #     pass
+    #
+    # def test_complete_in_create_index(self):
+    #     pass
+    #
+    # def test_complete_in_drop_index(self):
+    #     pass
 
-    def test_complete_in_truncate(self):
-        pass
-
-    def test_complete_in_alter_columnfamily(self):
-        pass
-
-    def test_complete_in_use(self):
-        pass
-
-    def test_complete_in_create_index(self):
-        pass
-
-    def test_complete_in_drop_index(self):
-        pass
+    def test_complete_in_alter_keyspace(self):
+        self.trycompletions('ALTER KEY', 'SPACE ')
+        self.trycompletions('ALTER KEYSPACE ', '', choices=[self.cqlsh.keyspace, 'system_auth',
+                                                            'system_distributed', 'system_traces'])
+        self.trycompletions('ALTER KEYSPACE system_trac', "es WITH replication = {'class': '")
+        self.trycompletions("ALTER KEYSPACE system_traces WITH replication = {'class': '", '',
+                            choices=['NetworkTopologyStrategy', 'SimpleStrategy'])
